/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.apache.geronimo.st.v11.core.jaxb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @version $Rev: 685643 $ $Date: 2008-08-14 03:24:11 +0800 (Thu, 14 Aug 2008) $ 
 */
public class NamespacePrefix {

    private static Map<String, String> prefixMap = new HashMap<String, String>();

    static {
        prefixMap.put("http://geronimo.apache.org/xml/ns/deployment-1.1", "dep");
        prefixMap.put("http://geronimo.apache.org/xml/ns/j2ee/application-1.1", "app");
        prefixMap.put("http://geronimo.apache.org/xml/ns/j2ee/application-client-1.1", "client");
        prefixMap.put("http://geronimo.apache.org/xml/ns/j2ee/connector-1.1", "conn");
        prefixMap.put("http://www.openejb.org/xml/ns/openejb-jar-2.1", "ejb");
        prefixMap.put("http://www.openejb.org/xml/ns/pkgen-2.0", "pkgen");
        prefixMap.put("http://geronimo.apache.org/xml/ns/naming-1.1", "name");
        prefixMap.put("http://geronimo.apache.org/xml/ns/security-1.1", "sec");
        prefixMap.put("http://geronimo.apache.org/xml/ns/j2ee/web-1.1", "web");
        prefixMap.put("http://www.openejb.org/xml/ns/corba-tss-config-2.0", "cbt");
        prefixMap.put("http://www.openejb.org/xml/ns/corba-css-config-2.0", "cbc");
    }
	
	public static void processPrefix( Node parent ) {
		NodeList nl = parent.getChildNodes();
		
		if ( parent instanceof Element ) {
			updatePrefix( (Element)parent );
		}
		
		for ( int i = 0; i <= nl.getLength(); i ++ ) {
			Node node = nl.item(i);
			if ( node instanceof Element ) {
				processPrefix( node );
			}
		}

	}
	
	private static void updatePrefix( Element element ) {
		NamedNodeMap mnm = element.getAttributes();
		
		ArrayList<Attr> attributes = new ArrayList<Attr>();
		for ( int j = 0; j <= mnm.getLength(); j ++ ) { 
			Attr attr = (Attr)mnm.item(j);
			if ( attr != null && attr.getOwnerElement() != null && getPrefix( attr.getNodeValue() ) != null ) {
				attributes.add((Attr)attr.cloneNode(false));
			}
		}
		for ( int j = 0; j < attributes.size(); j ++ ) {
			Attr tempAttr = attributes.get(j);
			Attr attr = element.getAttributeNode(tempAttr.getName());
			Element owner = (Element)attr.getOwnerElement();
			owner.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:" + getPrefix( attr.getNodeValue() ), attr.getNodeValue());
			owner.removeAttributeNode(attr);
		}
		String prefix = getPrefix( element.getNamespaceURI() );
		if ( prefix != null ) {
		    element.setPrefix( prefix );
		}
	}

	private static String getPrefix(String namespaceURI) {
        if (prefixMap.containsKey(namespaceURI))
            return prefixMap.get(namespaceURI);
		return null;
	}

}
